# coding: utf-8

"""
Marketcap Service API

API for retrieving historical marketcap data, available exchanges, and indicators.

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from typing import Any, Awaitable, Dict, List, Optional, Tuple, Union

from pydantic import Field, StrictFloat, StrictInt, StrictStr, validate_call
from typing_extensions import Annotated

from crypticorn.metrics.client.api_client import ApiClient, RequestSerialized
from crypticorn.metrics.client.api_response import ApiResponse
from crypticorn.metrics.client.models.exchange_availability import ExchangeAvailability
from crypticorn.metrics.client.models.exchange_mapping import ExchangeMapping
from crypticorn.metrics.client.models.internal_exchange import InternalExchange
from crypticorn.metrics.client.models.market_type import MarketType
from crypticorn.metrics.client.models.time_interval import TimeInterval
from crypticorn.metrics.client.models.trading_status import TradingStatus
from crypticorn.metrics.client.rest import RESTResponseType

# Import async_to_sync for sync methods
try:
    from asgiref.sync import async_to_sync

    _HAS_ASGIREF = True
except ImportError:
    _HAS_ASGIREF = False

    def async_to_sync(async_func):
        """Fallback decorator that raises an error if asgiref is not available."""

        def wrapper(*args, **kwargs):
            raise ImportError(
                "asgiref is required for sync methods. Install with: pip install asgiref"
            )

        return wrapper


class ExchangesApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None, is_sync: bool = False) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client
        self.is_sync = is_sync

    @validate_call
    def get_available_exchanges(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        symbol: Annotated[
            StrictStr, Field(description="Symbol to fetch available exchanges for")
        ],
        interval: Annotated[
            Optional[TimeInterval],
            Field(description="Interval for which to fetch available exchanges"),
        ] = None,
        start_timestamp: Annotated[
            Optional[StrictInt],
            Field(
                description="Start timestamp for which to fetch available exchanges (defaults to previous 7 day's closing)"
            ),
        ] = None,
        end_timestamp: Annotated[
            Optional[StrictInt],
            Field(description="End timestamp for which to fetch available exchanges"),
        ] = None,
        quote_currency: Annotated[
            Optional[StrictStr],
            Field(
                description="Quote currency for which to fetch available exchanges (Use quote currencies endpoint to get available quote currencies)"
            ),
        ] = None,
        status: Annotated[
            Optional[TradingStatus],
            Field(
                description="Trading pair status for which to fetch available exchanges"
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[List[ExchangeAvailability], Awaitable[List[ExchangeAvailability]]]:
        """Get Available Exchanges"""
        if self.is_sync:
            return self._get_available_exchanges_sync(
                market=market,
                symbol=symbol,
                interval=interval,
                start_timestamp=start_timestamp,
                end_timestamp=end_timestamp,
                quote_currency=quote_currency,
                status=status,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_available_exchanges_async(
                market=market,
                symbol=symbol,
                interval=interval,
                start_timestamp=start_timestamp,
                end_timestamp=end_timestamp,
                quote_currency=quote_currency,
                status=status,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    @validate_call
    def get_available_exchanges_with_http_info(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        symbol: Annotated[
            StrictStr, Field(description="Symbol to fetch available exchanges for")
        ],
        interval: Annotated[
            Optional[TimeInterval],
            Field(description="Interval for which to fetch available exchanges"),
        ] = None,
        start_timestamp: Annotated[
            Optional[StrictInt],
            Field(
                description="Start timestamp for which to fetch available exchanges (defaults to previous 7 day's closing)"
            ),
        ] = None,
        end_timestamp: Annotated[
            Optional[StrictInt],
            Field(description="End timestamp for which to fetch available exchanges"),
        ] = None,
        quote_currency: Annotated[
            Optional[StrictStr],
            Field(
                description="Quote currency for which to fetch available exchanges (Use quote currencies endpoint to get available quote currencies)"
            ),
        ] = None,
        status: Annotated[
            Optional[TradingStatus],
            Field(
                description="Trading pair status for which to fetch available exchanges"
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[
        ApiResponse[List[ExchangeAvailability]],
        Awaitable[ApiResponse[List[ExchangeAvailability]]],
    ]:
        """Get Available Exchanges with HTTP info"""
        if self.is_sync:
            return self._get_available_exchanges_sync_with_http_info(
                market=market,
                symbol=symbol,
                interval=interval,
                start_timestamp=start_timestamp,
                end_timestamp=end_timestamp,
                quote_currency=quote_currency,
                status=status,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_available_exchanges_async_with_http_info(
                market=market,
                symbol=symbol,
                interval=interval,
                start_timestamp=start_timestamp,
                end_timestamp=end_timestamp,
                quote_currency=quote_currency,
                status=status,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    @validate_call
    def get_available_exchanges_without_preload_content(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        symbol: Annotated[
            StrictStr, Field(description="Symbol to fetch available exchanges for")
        ],
        interval: Annotated[
            Optional[TimeInterval],
            Field(description="Interval for which to fetch available exchanges"),
        ] = None,
        start_timestamp: Annotated[
            Optional[StrictInt],
            Field(
                description="Start timestamp for which to fetch available exchanges (defaults to previous 7 day's closing)"
            ),
        ] = None,
        end_timestamp: Annotated[
            Optional[StrictInt],
            Field(description="End timestamp for which to fetch available exchanges"),
        ] = None,
        quote_currency: Annotated[
            Optional[StrictStr],
            Field(
                description="Quote currency for which to fetch available exchanges (Use quote currencies endpoint to get available quote currencies)"
            ),
        ] = None,
        status: Annotated[
            Optional[TradingStatus],
            Field(
                description="Trading pair status for which to fetch available exchanges"
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[RESTResponseType, Awaitable[RESTResponseType]]:
        """Get Available Exchanges without preloading content"""
        if self.is_sync:
            return self._get_available_exchanges_sync_without_preload_content(
                market=market,
                symbol=symbol,
                interval=interval,
                start_timestamp=start_timestamp,
                end_timestamp=end_timestamp,
                quote_currency=quote_currency,
                status=status,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_available_exchanges_async_without_preload_content(
                market=market,
                symbol=symbol,
                interval=interval,
                start_timestamp=start_timestamp,
                end_timestamp=end_timestamp,
                quote_currency=quote_currency,
                status=status,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    # Private async implementation methods
    @validate_call
    async def _get_available_exchanges_async(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        symbol: Annotated[
            StrictStr, Field(description="Symbol to fetch available exchanges for")
        ],
        interval: Annotated[
            Optional[TimeInterval],
            Field(description="Interval for which to fetch available exchanges"),
        ] = None,
        start_timestamp: Annotated[
            Optional[StrictInt],
            Field(
                description="Start timestamp for which to fetch available exchanges (defaults to previous 7 day's closing)"
            ),
        ] = None,
        end_timestamp: Annotated[
            Optional[StrictInt],
            Field(description="End timestamp for which to fetch available exchanges"),
        ] = None,
        quote_currency: Annotated[
            Optional[StrictStr],
            Field(
                description="Quote currency for which to fetch available exchanges (Use quote currencies endpoint to get available quote currencies)"
            ),
        ] = None,
        status: Annotated[
            Optional[TradingStatus],
            Field(
                description="Trading pair status for which to fetch available exchanges"
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[ExchangeAvailability]:
        """Get Available Exchanges

        Get available exchanges for a symbol with various filtering options.

        :param market: Market type (spot or futures) (required)
        :type market: MarketType
        :param symbol: Symbol to fetch available exchanges for (required)
        :type symbol: str
        :param interval: Interval for which to fetch available exchanges
        :type interval: TimeInterval
        :param start_timestamp: Start timestamp for which to fetch available exchanges (defaults to previous 7 day's closing)
        :type start_timestamp: int
        :param end_timestamp: End timestamp for which to fetch available exchanges
        :type end_timestamp: int
        :param quote_currency: Quote currency for which to fetch available exchanges (Use quote currencies endpoint to get available quote currencies)
        :type quote_currency: str
        :param status: Trading pair status for which to fetch available exchanges
        :type status: TradingStatus
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_available_exchanges_serialize(
            market=market,
            symbol=symbol,
            interval=interval,
            start_timestamp=start_timestamp,
            end_timestamp=end_timestamp,
            quote_currency=quote_currency,
            status=status,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[ExchangeAvailability]",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    async def _get_available_exchanges_async_with_http_info(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        symbol: Annotated[
            StrictStr, Field(description="Symbol to fetch available exchanges for")
        ],
        interval: Annotated[
            Optional[TimeInterval],
            Field(description="Interval for which to fetch available exchanges"),
        ] = None,
        start_timestamp: Annotated[
            Optional[StrictInt],
            Field(
                description="Start timestamp for which to fetch available exchanges (defaults to previous 7 day's closing)"
            ),
        ] = None,
        end_timestamp: Annotated[
            Optional[StrictInt],
            Field(description="End timestamp for which to fetch available exchanges"),
        ] = None,
        quote_currency: Annotated[
            Optional[StrictStr],
            Field(
                description="Quote currency for which to fetch available exchanges (Use quote currencies endpoint to get available quote currencies)"
            ),
        ] = None,
        status: Annotated[
            Optional[TradingStatus],
            Field(
                description="Trading pair status for which to fetch available exchanges"
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[ExchangeAvailability]]:
        """Get Available Exchanges

        Get available exchanges for a symbol with various filtering options.

        :param market: Market type (spot or futures) (required)
        :type market: MarketType
        :param symbol: Symbol to fetch available exchanges for (required)
        :type symbol: str
        :param interval: Interval for which to fetch available exchanges
        :type interval: TimeInterval
        :param start_timestamp: Start timestamp for which to fetch available exchanges (defaults to previous 7 day's closing)
        :type start_timestamp: int
        :param end_timestamp: End timestamp for which to fetch available exchanges
        :type end_timestamp: int
        :param quote_currency: Quote currency for which to fetch available exchanges (Use quote currencies endpoint to get available quote currencies)
        :type quote_currency: str
        :param status: Trading pair status for which to fetch available exchanges
        :type status: TradingStatus
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_available_exchanges_serialize(
            market=market,
            symbol=symbol,
            interval=interval,
            start_timestamp=start_timestamp,
            end_timestamp=end_timestamp,
            quote_currency=quote_currency,
            status=status,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[ExchangeAvailability]",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data, response_types_map=_response_types_map
        )

    @validate_call
    async def _get_available_exchanges_async_without_preload_content(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        symbol: Annotated[
            StrictStr, Field(description="Symbol to fetch available exchanges for")
        ],
        interval: Annotated[
            Optional[TimeInterval],
            Field(description="Interval for which to fetch available exchanges"),
        ] = None,
        start_timestamp: Annotated[
            Optional[StrictInt],
            Field(
                description="Start timestamp for which to fetch available exchanges (defaults to previous 7 day's closing)"
            ),
        ] = None,
        end_timestamp: Annotated[
            Optional[StrictInt],
            Field(description="End timestamp for which to fetch available exchanges"),
        ] = None,
        quote_currency: Annotated[
            Optional[StrictStr],
            Field(
                description="Quote currency for which to fetch available exchanges (Use quote currencies endpoint to get available quote currencies)"
            ),
        ] = None,
        status: Annotated[
            Optional[TradingStatus],
            Field(
                description="Trading pair status for which to fetch available exchanges"
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get Available Exchanges

        Get available exchanges for a symbol with various filtering options.

        :param market: Market type (spot or futures) (required)
        :type market: MarketType
        :param symbol: Symbol to fetch available exchanges for (required)
        :type symbol: str
        :param interval: Interval for which to fetch available exchanges
        :type interval: TimeInterval
        :param start_timestamp: Start timestamp for which to fetch available exchanges (defaults to previous 7 day's closing)
        :type start_timestamp: int
        :param end_timestamp: End timestamp for which to fetch available exchanges
        :type end_timestamp: int
        :param quote_currency: Quote currency for which to fetch available exchanges (Use quote currencies endpoint to get available quote currencies)
        :type quote_currency: str
        :param status: Trading pair status for which to fetch available exchanges
        :type status: TradingStatus
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_available_exchanges_serialize(
            market=market,
            symbol=symbol,
            interval=interval,
            start_timestamp=start_timestamp,
            end_timestamp=end_timestamp,
            quote_currency=quote_currency,
            status=status,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[ExchangeAvailability]",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data

    # Private sync implementation methods
    @validate_call
    def _get_available_exchanges_sync(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        symbol: Annotated[
            StrictStr, Field(description="Symbol to fetch available exchanges for")
        ],
        interval: Annotated[
            Optional[TimeInterval],
            Field(description="Interval for which to fetch available exchanges"),
        ] = None,
        start_timestamp: Annotated[
            Optional[StrictInt],
            Field(
                description="Start timestamp for which to fetch available exchanges (defaults to previous 7 day's closing)"
            ),
        ] = None,
        end_timestamp: Annotated[
            Optional[StrictInt],
            Field(description="End timestamp for which to fetch available exchanges"),
        ] = None,
        quote_currency: Annotated[
            Optional[StrictStr],
            Field(
                description="Quote currency for which to fetch available exchanges (Use quote currencies endpoint to get available quote currencies)"
            ),
        ] = None,
        status: Annotated[
            Optional[TradingStatus],
            Field(
                description="Trading pair status for which to fetch available exchanges"
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[ExchangeAvailability]:
        """Synchronous version of get_available_exchanges"""
        return async_to_sync(self._get_available_exchanges_async)(
            market=market,
            symbol=symbol,
            interval=interval,
            start_timestamp=start_timestamp,
            end_timestamp=end_timestamp,
            quote_currency=quote_currency,
            status=status,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    @validate_call
    def _get_available_exchanges_sync_with_http_info(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        symbol: Annotated[
            StrictStr, Field(description="Symbol to fetch available exchanges for")
        ],
        interval: Annotated[
            Optional[TimeInterval],
            Field(description="Interval for which to fetch available exchanges"),
        ] = None,
        start_timestamp: Annotated[
            Optional[StrictInt],
            Field(
                description="Start timestamp for which to fetch available exchanges (defaults to previous 7 day's closing)"
            ),
        ] = None,
        end_timestamp: Annotated[
            Optional[StrictInt],
            Field(description="End timestamp for which to fetch available exchanges"),
        ] = None,
        quote_currency: Annotated[
            Optional[StrictStr],
            Field(
                description="Quote currency for which to fetch available exchanges (Use quote currencies endpoint to get available quote currencies)"
            ),
        ] = None,
        status: Annotated[
            Optional[TradingStatus],
            Field(
                description="Trading pair status for which to fetch available exchanges"
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[ExchangeAvailability]]:
        """Synchronous version of get_available_exchanges_with_http_info"""
        return async_to_sync(self._get_available_exchanges_async_with_http_info)(
            market=market,
            symbol=symbol,
            interval=interval,
            start_timestamp=start_timestamp,
            end_timestamp=end_timestamp,
            quote_currency=quote_currency,
            status=status,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    @validate_call
    def _get_available_exchanges_sync_without_preload_content(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        symbol: Annotated[
            StrictStr, Field(description="Symbol to fetch available exchanges for")
        ],
        interval: Annotated[
            Optional[TimeInterval],
            Field(description="Interval for which to fetch available exchanges"),
        ] = None,
        start_timestamp: Annotated[
            Optional[StrictInt],
            Field(
                description="Start timestamp for which to fetch available exchanges (defaults to previous 7 day's closing)"
            ),
        ] = None,
        end_timestamp: Annotated[
            Optional[StrictInt],
            Field(description="End timestamp for which to fetch available exchanges"),
        ] = None,
        quote_currency: Annotated[
            Optional[StrictStr],
            Field(
                description="Quote currency for which to fetch available exchanges (Use quote currencies endpoint to get available quote currencies)"
            ),
        ] = None,
        status: Annotated[
            Optional[TradingStatus],
            Field(
                description="Trading pair status for which to fetch available exchanges"
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Synchronous version of get_available_exchanges_without_preload_content"""
        return async_to_sync(
            self._get_available_exchanges_async_without_preload_content
        )(
            market=market,
            symbol=symbol,
            interval=interval,
            start_timestamp=start_timestamp,
            end_timestamp=end_timestamp,
            quote_currency=quote_currency,
            status=status,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    def _get_available_exchanges_serialize(
        self,
        market,
        symbol,
        interval,
        start_timestamp,
        end_timestamp,
        quote_currency,
        status,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if market is not None:

            _query_params.append(("market", market.value))

        if symbol is not None:

            _query_params.append(("symbol", symbol))

        if interval is not None:

            _query_params.append(("interval", interval.value))

        if start_timestamp is not None:

            _query_params.append(("start_timestamp", start_timestamp))

        if end_timestamp is not None:

            _query_params.append(("end_timestamp", end_timestamp))

        if quote_currency is not None:

            _query_params.append(("quote_currency", quote_currency))

        if status is not None:

            _query_params.append(("status", status.value))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # authentication setting
        _auth_settings: List[str] = ["APIKeyHeader", "HTTPBearer"]

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/exchanges/available",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    def get_available_exchanges_for_market(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[List[str], Awaitable[List[str]]]:
        """Get Exchange List"""
        if self.is_sync:
            return self._get_available_exchanges_for_market_sync(
                market=market,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_available_exchanges_for_market_async(
                market=market,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    @validate_call
    def get_available_exchanges_for_market_with_http_info(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[ApiResponse[List[str]], Awaitable[ApiResponse[List[str]]]]:
        """Get Exchange List with HTTP info"""
        if self.is_sync:
            return self._get_available_exchanges_for_market_sync_with_http_info(
                market=market,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_available_exchanges_for_market_async_with_http_info(
                market=market,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    @validate_call
    def get_available_exchanges_for_market_without_preload_content(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[RESTResponseType, Awaitable[RESTResponseType]]:
        """Get Exchange List without preloading content"""
        if self.is_sync:
            return (
                self._get_available_exchanges_for_market_sync_without_preload_content(
                    market=market,
                    _request_timeout=_request_timeout,
                    _request_auth=_request_auth,
                    _content_type=_content_type,
                    _headers=_headers,
                    _host_index=_host_index,
                )
            )

        else:
            return (
                self._get_available_exchanges_for_market_async_without_preload_content(
                    market=market,
                    _request_timeout=_request_timeout,
                    _request_auth=_request_auth,
                    _content_type=_content_type,
                    _headers=_headers,
                    _host_index=_host_index,
                )
            )

    # Private async implementation methods
    @validate_call
    async def _get_available_exchanges_for_market_async(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[str]:
        """Get Exchange List

        Get list of exchanges for a market.

        :param market: Market type (spot or futures) (required)
        :type market: MarketType
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_available_exchanges_for_market_serialize(
            market=market,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[str]",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    async def _get_available_exchanges_for_market_async_with_http_info(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[str]]:
        """Get Exchange List

        Get list of exchanges for a market.

        :param market: Market type (spot or futures) (required)
        :type market: MarketType
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_available_exchanges_for_market_serialize(
            market=market,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[str]",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data, response_types_map=_response_types_map
        )

    @validate_call
    async def _get_available_exchanges_for_market_async_without_preload_content(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get Exchange List

        Get list of exchanges for a market.

        :param market: Market type (spot or futures) (required)
        :type market: MarketType
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_available_exchanges_for_market_serialize(
            market=market,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[str]",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data

    # Private sync implementation methods
    @validate_call
    def _get_available_exchanges_for_market_sync(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[str]:
        """Synchronous version of get_available_exchanges_for_market"""
        return async_to_sync(self._get_available_exchanges_for_market_async)(
            market=market,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    @validate_call
    def _get_available_exchanges_for_market_sync_with_http_info(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[str]]:
        """Synchronous version of get_available_exchanges_for_market_with_http_info"""
        return async_to_sync(
            self._get_available_exchanges_for_market_async_with_http_info
        )(
            market=market,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    @validate_call
    def _get_available_exchanges_for_market_sync_without_preload_content(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Synchronous version of get_available_exchanges_for_market_without_preload_content"""
        return async_to_sync(
            self._get_available_exchanges_for_market_async_without_preload_content
        )(
            market=market,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    def _get_available_exchanges_for_market_serialize(
        self,
        market,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if market is not None:

            _query_params.append(("market", market.value))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # authentication setting
        _auth_settings: List[str] = ["APIKeyHeader", "HTTPBearer"]

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/exchanges/list",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    def get_exchange_mappings(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        exchange: Annotated[
            Optional[InternalExchange],
            Field(description="Exchange name for which to fetch exchange mappings"),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[List[ExchangeMapping], Awaitable[List[ExchangeMapping]]]:
        """Get Exchange Mappings"""
        if self.is_sync:
            return self._get_exchange_mappings_sync(
                market=market,
                exchange=exchange,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_exchange_mappings_async(
                market=market,
                exchange=exchange,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    @validate_call
    def get_exchange_mappings_with_http_info(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        exchange: Annotated[
            Optional[InternalExchange],
            Field(description="Exchange name for which to fetch exchange mappings"),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[
        ApiResponse[List[ExchangeMapping]],
        Awaitable[ApiResponse[List[ExchangeMapping]]],
    ]:
        """Get Exchange Mappings with HTTP info"""
        if self.is_sync:
            return self._get_exchange_mappings_sync_with_http_info(
                market=market,
                exchange=exchange,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_exchange_mappings_async_with_http_info(
                market=market,
                exchange=exchange,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    @validate_call
    def get_exchange_mappings_without_preload_content(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        exchange: Annotated[
            Optional[InternalExchange],
            Field(description="Exchange name for which to fetch exchange mappings"),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[RESTResponseType, Awaitable[RESTResponseType]]:
        """Get Exchange Mappings without preloading content"""
        if self.is_sync:
            return self._get_exchange_mappings_sync_without_preload_content(
                market=market,
                exchange=exchange,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_exchange_mappings_async_without_preload_content(
                market=market,
                exchange=exchange,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    # Private async implementation methods
    @validate_call
    async def _get_exchange_mappings_async(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        exchange: Annotated[
            Optional[InternalExchange],
            Field(description="Exchange name for which to fetch exchange mappings"),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[ExchangeMapping]:
        """Get Exchange Mappings

        Get exchange mappings for a market with optional exchange name filter.

        :param market: Market type (spot or futures) (required)
        :type market: MarketType
        :param exchange: Exchange name for which to fetch exchange mappings
        :type exchange: InternalExchange
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_exchange_mappings_serialize(
            market=market,
            exchange=exchange,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[ExchangeMapping]",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    async def _get_exchange_mappings_async_with_http_info(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        exchange: Annotated[
            Optional[InternalExchange],
            Field(description="Exchange name for which to fetch exchange mappings"),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[ExchangeMapping]]:
        """Get Exchange Mappings

        Get exchange mappings for a market with optional exchange name filter.

        :param market: Market type (spot or futures) (required)
        :type market: MarketType
        :param exchange: Exchange name for which to fetch exchange mappings
        :type exchange: InternalExchange
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_exchange_mappings_serialize(
            market=market,
            exchange=exchange,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[ExchangeMapping]",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data, response_types_map=_response_types_map
        )

    @validate_call
    async def _get_exchange_mappings_async_without_preload_content(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        exchange: Annotated[
            Optional[InternalExchange],
            Field(description="Exchange name for which to fetch exchange mappings"),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get Exchange Mappings

        Get exchange mappings for a market with optional exchange name filter.

        :param market: Market type (spot or futures) (required)
        :type market: MarketType
        :param exchange: Exchange name for which to fetch exchange mappings
        :type exchange: InternalExchange
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_exchange_mappings_serialize(
            market=market,
            exchange=exchange,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[ExchangeMapping]",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data

    # Private sync implementation methods
    @validate_call
    def _get_exchange_mappings_sync(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        exchange: Annotated[
            Optional[InternalExchange],
            Field(description="Exchange name for which to fetch exchange mappings"),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[ExchangeMapping]:
        """Synchronous version of get_exchange_mappings"""
        return async_to_sync(self._get_exchange_mappings_async)(
            market=market,
            exchange=exchange,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    @validate_call
    def _get_exchange_mappings_sync_with_http_info(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        exchange: Annotated[
            Optional[InternalExchange],
            Field(description="Exchange name for which to fetch exchange mappings"),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[ExchangeMapping]]:
        """Synchronous version of get_exchange_mappings_with_http_info"""
        return async_to_sync(self._get_exchange_mappings_async_with_http_info)(
            market=market,
            exchange=exchange,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    @validate_call
    def _get_exchange_mappings_sync_without_preload_content(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        exchange: Annotated[
            Optional[InternalExchange],
            Field(description="Exchange name for which to fetch exchange mappings"),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Synchronous version of get_exchange_mappings_without_preload_content"""
        return async_to_sync(self._get_exchange_mappings_async_without_preload_content)(
            market=market,
            exchange=exchange,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    def _get_exchange_mappings_serialize(
        self,
        market,
        exchange,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if market is not None:

            _query_params.append(("market", market.value))

        if exchange is not None:

            _query_params.append(("exchange", exchange.value))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # authentication setting
        _auth_settings: List[str] = ["APIKeyHeader", "HTTPBearer"]

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/exchanges/mappings",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    def get_exchange_mappings_for_pair(
        self,
        pair: Annotated[
            StrictStr,
            Field(
                description="Trading pair to find mappings for. Can be exchange specific (e.g., 'BTCUSDT', '1000SHIBUSDT') or a common format Symbol/QuoteCurrency (e.g., 'BTC/USDT')"
            ),
        ],
        market: Annotated[
            Optional[MarketType],
            Field(description="Market type (spot or futures). Defaults to futures."),
        ] = None,
        exchange: Annotated[
            Optional[InternalExchange],
            Field(
                description="Optional exchange name to filter by. If not provided, returns all exchanges a match was found for."
            ),
        ] = None,
        quote_currency: Annotated[
            Optional[StrictStr],
            Field(description="Quote currency to filter by. Defaults to USDT."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[List[ExchangeMapping], Awaitable[List[ExchangeMapping]]]:
        """Get Exchange Mappings For Pair"""
        if self.is_sync:
            return self._get_exchange_mappings_for_pair_sync(
                pair=pair,
                market=market,
                exchange=exchange,
                quote_currency=quote_currency,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_exchange_mappings_for_pair_async(
                pair=pair,
                market=market,
                exchange=exchange,
                quote_currency=quote_currency,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    @validate_call
    def get_exchange_mappings_for_pair_with_http_info(
        self,
        pair: Annotated[
            StrictStr,
            Field(
                description="Trading pair to find mappings for. Can be exchange specific (e.g., 'BTCUSDT', '1000SHIBUSDT') or a common format Symbol/QuoteCurrency (e.g., 'BTC/USDT')"
            ),
        ],
        market: Annotated[
            Optional[MarketType],
            Field(description="Market type (spot or futures). Defaults to futures."),
        ] = None,
        exchange: Annotated[
            Optional[InternalExchange],
            Field(
                description="Optional exchange name to filter by. If not provided, returns all exchanges a match was found for."
            ),
        ] = None,
        quote_currency: Annotated[
            Optional[StrictStr],
            Field(description="Quote currency to filter by. Defaults to USDT."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[
        ApiResponse[List[ExchangeMapping]],
        Awaitable[ApiResponse[List[ExchangeMapping]]],
    ]:
        """Get Exchange Mappings For Pair with HTTP info"""
        if self.is_sync:
            return self._get_exchange_mappings_for_pair_sync_with_http_info(
                pair=pair,
                market=market,
                exchange=exchange,
                quote_currency=quote_currency,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_exchange_mappings_for_pair_async_with_http_info(
                pair=pair,
                market=market,
                exchange=exchange,
                quote_currency=quote_currency,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    @validate_call
    def get_exchange_mappings_for_pair_without_preload_content(
        self,
        pair: Annotated[
            StrictStr,
            Field(
                description="Trading pair to find mappings for. Can be exchange specific (e.g., 'BTCUSDT', '1000SHIBUSDT') or a common format Symbol/QuoteCurrency (e.g., 'BTC/USDT')"
            ),
        ],
        market: Annotated[
            Optional[MarketType],
            Field(description="Market type (spot or futures). Defaults to futures."),
        ] = None,
        exchange: Annotated[
            Optional[InternalExchange],
            Field(
                description="Optional exchange name to filter by. If not provided, returns all exchanges a match was found for."
            ),
        ] = None,
        quote_currency: Annotated[
            Optional[StrictStr],
            Field(description="Quote currency to filter by. Defaults to USDT."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[RESTResponseType, Awaitable[RESTResponseType]]:
        """Get Exchange Mappings For Pair without preloading content"""
        if self.is_sync:
            return self._get_exchange_mappings_for_pair_sync_without_preload_content(
                pair=pair,
                market=market,
                exchange=exchange,
                quote_currency=quote_currency,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_exchange_mappings_for_pair_async_without_preload_content(
                pair=pair,
                market=market,
                exchange=exchange,
                quote_currency=quote_currency,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    # Private async implementation methods
    @validate_call
    async def _get_exchange_mappings_for_pair_async(
        self,
        pair: Annotated[
            StrictStr,
            Field(
                description="Trading pair to find mappings for. Can be exchange specific (e.g., 'BTCUSDT', '1000SHIBUSDT') or a common format Symbol/QuoteCurrency (e.g., 'BTC/USDT')"
            ),
        ],
        market: Annotated[
            Optional[MarketType],
            Field(description="Market type (spot or futures). Defaults to futures."),
        ] = None,
        exchange: Annotated[
            Optional[InternalExchange],
            Field(
                description="Optional exchange name to filter by. If not provided, returns all exchanges a match was found for."
            ),
        ] = None,
        quote_currency: Annotated[
            Optional[StrictStr],
            Field(description="Quote currency to filter by. Defaults to USDT."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[ExchangeMapping]:
        """Get Exchange Mappings For Pair

        Get exchange mappings for a specific trading pair across all exchanges or a specific exchange. This endpoint finds the underlying symbol for a given pair and returns all perpetual contracts (excluding quarterly/dated contracts) with the specified quote currency.

        :param pair: Trading pair to find mappings for. Can be exchange specific (e.g., 'BTCUSDT', '1000SHIBUSDT') or a common format Symbol/QuoteCurrency (e.g., 'BTC/USDT') (required)
        :type pair: str
        :param market: Market type (spot or futures). Defaults to futures.
        :type market: MarketType
        :param exchange: Optional exchange name to filter by. If not provided, returns all exchanges a match was found for.
        :type exchange: InternalExchange
        :param quote_currency: Quote currency to filter by. Defaults to USDT.
        :type quote_currency: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_exchange_mappings_for_pair_serialize(
            pair=pair,
            market=market,
            exchange=exchange,
            quote_currency=quote_currency,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[ExchangeMapping]",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    async def _get_exchange_mappings_for_pair_async_with_http_info(
        self,
        pair: Annotated[
            StrictStr,
            Field(
                description="Trading pair to find mappings for. Can be exchange specific (e.g., 'BTCUSDT', '1000SHIBUSDT') or a common format Symbol/QuoteCurrency (e.g., 'BTC/USDT')"
            ),
        ],
        market: Annotated[
            Optional[MarketType],
            Field(description="Market type (spot or futures). Defaults to futures."),
        ] = None,
        exchange: Annotated[
            Optional[InternalExchange],
            Field(
                description="Optional exchange name to filter by. If not provided, returns all exchanges a match was found for."
            ),
        ] = None,
        quote_currency: Annotated[
            Optional[StrictStr],
            Field(description="Quote currency to filter by. Defaults to USDT."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[ExchangeMapping]]:
        """Get Exchange Mappings For Pair

        Get exchange mappings for a specific trading pair across all exchanges or a specific exchange. This endpoint finds the underlying symbol for a given pair and returns all perpetual contracts (excluding quarterly/dated contracts) with the specified quote currency.

        :param pair: Trading pair to find mappings for. Can be exchange specific (e.g., 'BTCUSDT', '1000SHIBUSDT') or a common format Symbol/QuoteCurrency (e.g., 'BTC/USDT') (required)
        :type pair: str
        :param market: Market type (spot or futures). Defaults to futures.
        :type market: MarketType
        :param exchange: Optional exchange name to filter by. If not provided, returns all exchanges a match was found for.
        :type exchange: InternalExchange
        :param quote_currency: Quote currency to filter by. Defaults to USDT.
        :type quote_currency: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_exchange_mappings_for_pair_serialize(
            pair=pair,
            market=market,
            exchange=exchange,
            quote_currency=quote_currency,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[ExchangeMapping]",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data, response_types_map=_response_types_map
        )

    @validate_call
    async def _get_exchange_mappings_for_pair_async_without_preload_content(
        self,
        pair: Annotated[
            StrictStr,
            Field(
                description="Trading pair to find mappings for. Can be exchange specific (e.g., 'BTCUSDT', '1000SHIBUSDT') or a common format Symbol/QuoteCurrency (e.g., 'BTC/USDT')"
            ),
        ],
        market: Annotated[
            Optional[MarketType],
            Field(description="Market type (spot or futures). Defaults to futures."),
        ] = None,
        exchange: Annotated[
            Optional[InternalExchange],
            Field(
                description="Optional exchange name to filter by. If not provided, returns all exchanges a match was found for."
            ),
        ] = None,
        quote_currency: Annotated[
            Optional[StrictStr],
            Field(description="Quote currency to filter by. Defaults to USDT."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get Exchange Mappings For Pair

        Get exchange mappings for a specific trading pair across all exchanges or a specific exchange. This endpoint finds the underlying symbol for a given pair and returns all perpetual contracts (excluding quarterly/dated contracts) with the specified quote currency.

        :param pair: Trading pair to find mappings for. Can be exchange specific (e.g., 'BTCUSDT', '1000SHIBUSDT') or a common format Symbol/QuoteCurrency (e.g., 'BTC/USDT') (required)
        :type pair: str
        :param market: Market type (spot or futures). Defaults to futures.
        :type market: MarketType
        :param exchange: Optional exchange name to filter by. If not provided, returns all exchanges a match was found for.
        :type exchange: InternalExchange
        :param quote_currency: Quote currency to filter by. Defaults to USDT.
        :type quote_currency: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_exchange_mappings_for_pair_serialize(
            pair=pair,
            market=market,
            exchange=exchange,
            quote_currency=quote_currency,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[ExchangeMapping]",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data

    # Private sync implementation methods
    @validate_call
    def _get_exchange_mappings_for_pair_sync(
        self,
        pair: Annotated[
            StrictStr,
            Field(
                description="Trading pair to find mappings for. Can be exchange specific (e.g., 'BTCUSDT', '1000SHIBUSDT') or a common format Symbol/QuoteCurrency (e.g., 'BTC/USDT')"
            ),
        ],
        market: Annotated[
            Optional[MarketType],
            Field(description="Market type (spot or futures). Defaults to futures."),
        ] = None,
        exchange: Annotated[
            Optional[InternalExchange],
            Field(
                description="Optional exchange name to filter by. If not provided, returns all exchanges a match was found for."
            ),
        ] = None,
        quote_currency: Annotated[
            Optional[StrictStr],
            Field(description="Quote currency to filter by. Defaults to USDT."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[ExchangeMapping]:
        """Synchronous version of get_exchange_mappings_for_pair"""
        return async_to_sync(self._get_exchange_mappings_for_pair_async)(
            pair=pair,
            market=market,
            exchange=exchange,
            quote_currency=quote_currency,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    @validate_call
    def _get_exchange_mappings_for_pair_sync_with_http_info(
        self,
        pair: Annotated[
            StrictStr,
            Field(
                description="Trading pair to find mappings for. Can be exchange specific (e.g., 'BTCUSDT', '1000SHIBUSDT') or a common format Symbol/QuoteCurrency (e.g., 'BTC/USDT')"
            ),
        ],
        market: Annotated[
            Optional[MarketType],
            Field(description="Market type (spot or futures). Defaults to futures."),
        ] = None,
        exchange: Annotated[
            Optional[InternalExchange],
            Field(
                description="Optional exchange name to filter by. If not provided, returns all exchanges a match was found for."
            ),
        ] = None,
        quote_currency: Annotated[
            Optional[StrictStr],
            Field(description="Quote currency to filter by. Defaults to USDT."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[ExchangeMapping]]:
        """Synchronous version of get_exchange_mappings_for_pair_with_http_info"""
        return async_to_sync(self._get_exchange_mappings_for_pair_async_with_http_info)(
            pair=pair,
            market=market,
            exchange=exchange,
            quote_currency=quote_currency,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    @validate_call
    def _get_exchange_mappings_for_pair_sync_without_preload_content(
        self,
        pair: Annotated[
            StrictStr,
            Field(
                description="Trading pair to find mappings for. Can be exchange specific (e.g., 'BTCUSDT', '1000SHIBUSDT') or a common format Symbol/QuoteCurrency (e.g., 'BTC/USDT')"
            ),
        ],
        market: Annotated[
            Optional[MarketType],
            Field(description="Market type (spot or futures). Defaults to futures."),
        ] = None,
        exchange: Annotated[
            Optional[InternalExchange],
            Field(
                description="Optional exchange name to filter by. If not provided, returns all exchanges a match was found for."
            ),
        ] = None,
        quote_currency: Annotated[
            Optional[StrictStr],
            Field(description="Quote currency to filter by. Defaults to USDT."),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Synchronous version of get_exchange_mappings_for_pair_without_preload_content"""
        return async_to_sync(
            self._get_exchange_mappings_for_pair_async_without_preload_content
        )(
            pair=pair,
            market=market,
            exchange=exchange,
            quote_currency=quote_currency,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    def _get_exchange_mappings_for_pair_serialize(
        self,
        pair,
        market,
        exchange,
        quote_currency,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if pair is not None:

            _query_params.append(("pair", pair))

        if market is not None:

            _query_params.append(("market", market.value))

        if exchange is not None:

            _query_params.append(("exchange", exchange.value))

        if quote_currency is not None:

            _query_params.append(("quote_currency", quote_currency))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # authentication setting
        _auth_settings: List[str] = ["APIKeyHeader", "HTTPBearer"]

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/exchanges/pair-mappings",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )



# coding: utf-8

"""
Trading API

API for automated trading and exchange interface. This API is used to trade on the exchange and manage bots, API keys, orders, and more.

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, StrictBool
from typing_extensions import Self


class NotificationUpdate(BaseModel):
    """
    Notification model for API update operations.
    """  # noqa: E501

    viewed: Optional[StrictBool] = None
    sent: Optional[StrictBool] = None
    __properties: ClassVar[List[str]] = ["viewed", "sent"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of NotificationUpdate from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if viewed (nullable) is None
        # and model_fields_set contains the field
        if self.viewed is None and "viewed" in self.model_fields_set:
            _dict["viewed"] = None

        # set to None if sent (nullable) is None
        # and model_fields_set contains the field
        if self.sent is None and "sent" in self.model_fields_set:
            _dict["sent"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of NotificationUpdate from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {"viewed": obj.get("viewed"), "sent": obj.get("sent")}
        )
        return _obj



# coding: utf-8

"""
Crypticorn Auth API

OpenAPI compliant REST API built using tRPC with Express

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing_extensions import Self


class CreateUserWithWalletRequest(BaseModel):
    """
    CreateUserWithWalletRequest
    """  # noqa: E501

    oob_code: StrictStr = Field(
        description="OOB Code returned from walletNonce. This method must be called before creating a user with a wallet.",
        alias="oobCode",
    )
    signature: StrictStr = Field(
        description="Signature returned from the EVM client when signing the message returned from walletNonce. This method must be called before creating a user with a wallet."
    )
    message: StrictStr = Field(
        description="Message returned from walletNonce. This method must be called before creating a user with a wallet."
    )
    username: Optional[StrictStr] = Field(
        default=None, description="Username for the user"
    )
    name: Optional[StrictStr] = Field(
        default=None, description="Display name for the user"
    )
    picture: Optional[StrictStr] = Field(
        default=None, description="Profile picture for the user"
    )
    phone: Optional[StrictStr] = Field(
        default=None, description="Phone number for the user"
    )
    __properties: ClassVar[List[str]] = [
        "oobCode",
        "signature",
        "message",
        "username",
        "name",
        "picture",
        "phone",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CreateUserWithWalletRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CreateUserWithWalletRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "oobCode": obj.get("oobCode"),
                "signature": obj.get("signature"),
                "message": obj.get("message"),
                "username": obj.get("username"),
                "name": obj.get("name"),
                "picture": obj.get("picture"),
                "phone": obj.get("phone"),
            }
        )
        return _obj



# coding: utf-8

"""
Trading API

API for automated trading and exchange interface. This API is used to trade on the exchange and manage bots, API keys, orders, and more.

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr
from typing_extensions import Self

from crypticorn.trade.client.models.exchange import Exchange


class ExchangeInfo(BaseModel):
    """
    Model for exchange info
    """  # noqa: E501

    exchange: Exchange = Field(description="Exchange")
    is_tradeable: StrictBool = Field(
        description="Whether the exchange is tradeable (trade execution is implemented)"
    )
    is_beta: StrictBool = Field(
        description="Whether the exchange is in public beta testing (marked as beta in the UI)"
    )
    is_public: StrictBool = Field(
        description="Whether the exchange is public (shown in the UI)"
    )
    is_planned: StrictBool = Field(
        description="Whether the exchange is planned to be added (marked as planned but disabled in the UI)"
    )
    is_deprecated: StrictBool = Field(
        description="Whether the exchange is deprecated (not supported anymore)"
    )
    ref_link: Optional[StrictStr] = None
    __properties: ClassVar[List[str]] = [
        "exchange",
        "is_tradeable",
        "is_beta",
        "is_public",
        "is_planned",
        "is_deprecated",
        "ref_link",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ExchangeInfo from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if ref_link (nullable) is None
        # and model_fields_set contains the field
        if self.ref_link is None and "ref_link" in self.model_fields_set:
            _dict["ref_link"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ExchangeInfo from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "exchange": obj.get("exchange"),
                "is_tradeable": obj.get("is_tradeable"),
                "is_beta": obj.get("is_beta"),
                "is_public": obj.get("is_public"),
                "is_planned": obj.get("is_planned"),
                "is_deprecated": obj.get("is_deprecated"),
                "ref_link": obj.get("ref_link"),
            }
        )
        return _obj
